import numpy as np
import matplotlib.pyplot as plt
from scipy.fft import fft, fftfreq
# (Ensure you import your InfluxDBClient if not already imported)
from influxdb_client import InfluxDBClient

# InfluxDB connection details
url = "http://localhost:8086"
token = "WUxftnono0_k_t620srsO7xNG15xcej5meoShrr1ONHGvWSEqwg3gJVhthKwux7wUyw1_1hm9TAQFWKeEBHK2g=="
org = "Student"

# Function to fetch and process data
def fetch_and_process_data(bucket, measurement, filter_value, filter_field="url_hostname"):
    with InfluxDBClient(url=url, token=token, org=org) as client:
        query_api = client.query_api()
        query = f'''
        from(bucket: "{bucket}")
          |> range(start: 2023-08-01T00:00:00Z, stop: 2023-08-02T00:00:00Z)
          |> filter(fn: (r) => r["_measurement"] == "{measurement}")
          |> filter(fn: (r) => r["{filter_field}"] == "{filter_value}")
          |> keep(columns: ["_time"])
        '''
        timestamps = [record.get_time() for table in query_api.query(query) for record in table.records]
        if not timestamps:
            return None, None
        time_grid = np.arange(0, (timestamps[-1] - timestamps[0]).total_seconds() + 1, 1)
        values = np.zeros_like(time_grid, dtype=float)
        for t in timestamps:
            values[int((t - timestamps[0]).total_seconds())] = 1
    return time_grid, values

# Fetch data for all buckets
time_grid1, values1 = fetch_and_process_data("Net", "hostnames", "saml.allianz.com")
time_grid2, values2 = fetch_and_process_data("Net8", "beacon_activity", "example1.beacon.com", filter_field="url")
time_grid3, values3 = fetch_and_process_data("Net9", "hostnames", "m4v4r4c5.stackpathcdn.com")

# Function to apply Fourier Transform with color adjustments
def apply_fourier_transform(time_grid, values, label, color, linestyle='-', linewidth=2, alpha=1.0):
    freqs = fftfreq(len(values), d=1.0)
    amplitudes = np.abs(fft(values)[:len(values)//2])

    # Apply amplitude limit only for saml.allianz.com
    if label == "(saml.allianz.com)":
        amplitudes = np.minimum(amplitudes, 50)

    plt.plot(freqs[:len(values)//2], amplitudes, label=label, color=color, linestyle=linestyle, linewidth=linewidth, alpha=alpha)
    plt.ylim(0, 600)

# Plot combined FFT results
plt.figure(figsize=(10, 6))

apply_fourier_transform(time_grid1, values1, "(saml.allianz.com)", "red", linewidth=3)  # Red and thick
apply_fourier_transform(time_grid2, values2, "(example1.beacon.com)", "lightblue", alpha=0.5)  # Light blue, dashed, semi-transparent
apply_fourier_transform(time_grid3, values3, "(m4v4r4c5.stackpathcdn.com)", "green", alpha=0.5)

plt.title("Combined Frequency Spectrum (Fourier Transform)")
plt.xlabel("Frequency (Hz)")
plt.ylabel("Amplitude")
plt.legend()
plt.grid()
plt.show()

# Function to apply Autocorrelation with adjusted visualization
def apply_autocorrelation(values, label, color, linestyle='-', linewidth=2, alpha=1.0):
    autocorr = np.correlate(values, values, mode='full')[len(values) - 1:]
    autocorr = autocorr / np.max(autocorr)  # Normalize autocorrelation values
    lags = np.arange(len(autocorr))
    plt.plot(lags, autocorr, label=label, color=color, linestyle=linestyle, linewidth=linewidth, alpha=alpha)

# Plot combined Autocorrelation results with adjusted colors and styles
plt.figure(figsize=(10, 6))

apply_autocorrelation(values1, "(saml.allianz.com)", "red", linewidth=3)  # Thicker and red to stand out
apply_autocorrelation(values2, "(example1.beacon.com)", "lightblue", alpha=0.5)  # Light color, dashed, transparent
apply_autocorrelation(values3, "(m4v4r4c5.stackpathcdn.com)", "green", alpha=0.5)

plt.title("Combined Autocorrelation Function")
plt.xlabel("Sequence Offset")
plt.ylabel("Autocorrelation")
plt.legend()
plt.grid()
plt.show()

